'
'Licensed Materials - Use restricted, please refer to the "Samples Gallery" terms and conditions in the IBM International Program License Agreement.
'
' Copyright IBM Corporation 2003, 2004. All Rights Reserved. 
'

Imports System 
Imports System.Windows.Forms
Imports Rational.Test.Ft.Enabler
Imports Rational.Test.Ft.Object.Interfaces
Imports Rational.Test.Ft.Script
Imports Rational.Test.Ft.Util
Imports Rational.Test.Ft.Value
Imports Rational.Test.Ft.Vp

Namespace Superscript
    MustInherit Public Class ExtensionScript
        Inherits RationalTestScript
        Public Function GetScriptName() As String



            Dim ISI As Rational.Test.Ft.Script.Impl.IScriptInternal = Me
            Return ISI.GetScriptName()   'Me.GetType().Name  

        End Function

        ' displays a OK|Cancel dialog box with specified text
        ' returns DialogResult.OK or DialogResult.Cancel
        Public Overridable Function MsgBoxConfirm(ByVal Message As String) As DialogResult
            Return MessageBox.Show(Message, GetScriptName(), MessageBoxButtons.OKCancel, MessageBoxIcon.Question)
        End Function

        ' displays a Yes|No|Cancel dialog box with specified text
        ' returns DialogResult.Yes, DialogResult.No, or DialogResult.Cancel
        Public Overridable Function MsgBoxInput(ByVal Message As String) As DialogResult
            Return MessageBox.Show(Message, GetScriptName(), MessageBoxButtons.YesNoCancel, MessageBoxIcon.Question)
        End Function

        ' displays a simple dialog box with specified text
        Public Overridable Sub MsgBox(ByVal Message As String)
            MessageBox.Show(Message, GetScriptName(), MessageBoxButtons.OK, MessageBoxIcon.Information)
        End Sub

        ' return the text on the clipboard.
        ' If there is no text, it returns an empty string	    
        Public Shared Function GetClipboardText() As String
            Dim Text As String
            Text = ""
            Dim ClipObject As System.Windows.Forms.IDataObject
            ClipObject = System.Windows.Forms.Clipboard.GetDataObject()

            If Not (ClipObject Is Nothing) Then
                If ClipObject.GetDataPresent("UnicodeText") Then
                    Text = ClipObject.GetData("UnicodeText")
                End If
            End If

            Return Text
        End Function

        ' Set the contents of the clipboard to the provided text
        Public Shared Sub SetClipBoardText(ByVal Text As String)
            System.Windows.Forms.Clipboard.SetDataObject(Text)
        End Sub

        ' A simple text vp for the clipboard.  If you insert a call to this method
        ' in your test script, the first time the script is run, the clipboard text will
        ' be captured and written to a VP baseline named by <code>vpName</code>. After
        ' the baseline is created on the first playback, running the script will cause the
        ' clipboard text to be captured, compared to the baseline, and the result will
        ' be written to the test log.
        Public Overridable Sub ClipboardVP(ByVal vpName As String)
            VpManual(vpName, GetClipboardText()).PerformTest()
        End Sub

        ' A simple timer class - it can be called in a polling loop to determine
        ' when your time is up.
        Public Class Timer

            Private StartTime As Long

            ' constructor - starts the timer.		    
            Public Sub New()
                Reset()
            End Sub

            ' Reset the timer to the current time.		    
            Public Overridable Sub Reset()
                Dim Time As System.DateTime
                Time = System.DateTime.Now
                StartTime = Time.Ticks
            End Sub

            ' Returns elapsed time (since timer was created) in seconds
            Public Overridable Function ElapsedTime() As Double
                Dim EndTime As Long
                Dim Time As System.DateTime
                Time = System.DateTime.Now
                EndTime = Time.Ticks
                Return (EndTime - StartTime) / 10000000
            End Function

            ' Returns true if elapsed time (since timer was created) 
            ' is greater than seconds		    
            Public Overridable Function IsTimeUp(ByVal Seconds As Double) As Boolean
                Return ElapsedTime() > Seconds
            End Function

        End Class 'Timer


        ' Wait for an object with default max time and delay between retries. 
        ' The finder interface is called to determine if the object is yet found.		
        Public Overridable Function WaitForObject(ByVal finder As IFindObject) As System.Object
            ' use the standard option delay times
            Dim maxTime As Double
            maxTime = GetOption(IOptionName.MAXIMUM_FIND_OBJECT_TIME)
            Dim delay As Double
            delay = GetOption(IOptionName.FIND_OBJECT_DELAY_BETWEEN_RETRIES)

            Return WaitForObject(finder, maxTime, delay)
        End Function

        ' Wait for an object with specified max time and delay between retries.
        ' As this method uses the RationalTestScript.sleep() method, it can be
        ' interrupted by the user hitting {F11}.
        ' The finder interface is called to determine if the object is yet found.		
        Public Overridable Function WaitForObject(ByVal finder As IFindObject, _
                                                        ByVal maxTime As Double, _
                                                        ByVal delay As Double) As System.Object
            Dim t As New Timer
            While (Not t.IsTimeUp(maxTime))
                Dim obj As System.Object
                obj = finder.FindObject()
                If Not (obj Is Nothing) Then
                    Return obj
                End If
                Sleep(delay)
            End While
            Dim ofinder As System.Object = finder
            Throw New ExtensionScriptException("waitForObject timed out: " + ofinder.ToString())
        End Function

        ' Wait for a vp with specified max time and delay between retries.
        ' As this method uses the RationalTestScript.Sleep() method, it can be
        ' interrupted by the user hitting {F11}.
        ' The IFtVerificationPoint.compare() method is called to determine if the VP passes.
        ' Setting the compareTrueEqualsPass parameter to false can reverse the meaning of the VP 
        ' The returned object contains the stored results. If you then invokes Compare()
        ' on the returned object, you will get the boolean result (pass or fail).  If you
        ' invoke PerformTest() on the returned object, the final result will be logged. Note
        ' that the returned object will not be affected by
        ' the compareTrueEqualsPass parameter.  To invert the logging, you must
        ' pass it in to PerformTest() as there is a version of performTest that
        ' takes the same parameter. 
        Public Function WaitForVp(ByVal vp As IFtVerificationPoint, _
                                   ByVal compareTrueEqualsPass As Boolean, _
                                   ByVal maxTime As Double, _
                                   ByVal delay As Double) As IFtVerificationPoint
            Dim t As Timer = New Timer
            While Not t.IsTimeUp(maxTime)
                ' internal method - this will be supported on IFtVerificationPoint
                ' in a future release
                Dim internalVp As Rational.Test.Ft.Vp.Impl.FtVerificationPoint = vp
                internalVp.Capture()
                Dim b As Boolean = internalVp.Compare()
                If b = compareTrueEqualsPass Then
                    Return internalVp
                End If
                Sleep(delay)
            End While
            Return vp
        End Function

        ' Wait for an VP with default max time and delay between retries. 
        ' The IFtVerificationPoint.Compare() method is called to determine if the VP passes.
        ' Setting the compareTrueEqualsPass parameter to false can reverse the meaning of the VP.	
        Public Function WaitForVp(ByVal vp As IFtVerificationPoint, ByVal compareTrueEqualsPass As Boolean) As IFtVerificationPoint
            ' use the standard option delay times
            Dim maxTime As Double = GetOption(IOptionName.MAXIMUM_FIND_OBJECT_TIME)
            Dim delay As Double = GetOption(IOptionName.FIND_OBJECT_DELAY_BETWEEN_RETRIES)
            Return WaitForVp(vp, compareTrueEqualsPass, maxTime, delay)
        End Function

        ' Wait for an VP with default max time and delay between retries. 
        ' The IFtVerificationPoint.compare() method is called to determine if the VP passes.
        Public Function WaitForVp(ByVal vp As IFtVerificationPoint) As IFtVerificationPoint
            Return WaitForVp(vp, True)
        End Function

        ' Wait for an VP with default max time and delay between retries. 
        ' The IFtVerificationPoint.compare() method is called to determine if the VP passes.
        Public Function WaitForVpFail(ByVal vp As IFtVerificationPoint) As IFtVerificationPoint
            Return WaitForVp(vp, False)
        End Function

        ' interface definition for an object finder. 
        Public Interface IFindObject
            ' returns an object if it can be found, returns null when no
            ' object is found				
            Function FindObject() As System.Object
        End Interface

        ' General-purpose exception for extension scripts	
        Public Class ExtensionScriptException
            Inherits System.Exception
            Sub New(ByVal s As System.String)
                MyBase.New(s)
            End Sub
        End Class 'ExtensionScriptException  

        ' pretty-print a hashtable (sorted)		
        Public Shared Sub PrintHash(ByVal h As System.Collections.Hashtable)
            Dim sl As New System.Collections.SortedList(h)
            Dim i As Integer
            For i = 0 To sl.Count - 1
                Dim Value As String = ""
                If Not (sl.GetByIndex(i) Is Nothing) Then
                    Value = sl.GetByIndex(i).ToString()
                End If
                Console.WriteLine(sl.GetKey(i).ToString() + " = " + Value)
            Next i
        End Sub

        ' print out the methods of an object (omitting methods from java.lang.Object)		
        Public Shared Sub PrintMethods(ByVal obj As TestObject)
            Dim m As MethodInfo() = obj.GetMethods()
            Dim i As Integer
            For i = 0 To m.Length - 1
                Dim clzz As String = m(i).GetDeclaringClass()
                Dim b As Boolean = clzz.Equals("System.Object")
                If Not b Then
                    Console.WriteLine(m(i).ToString())
                End If
            Next i
        End Sub

        ' Print the properties of a TestObject.
        Public Overridable Sub PrintProperties(ByVal tobj As TestObject)
            Dim table As System.Collections.Hashtable = tobj.GetProperties()
            If Not (table.Keys Is Nothing) Then
                Dim keysArray(table.Keys.Count) As System.Object
                table.Keys.CopyTo(keysArray, 0)
                Dim count As Integer
                For count = 0 To table.Keys.Count - 1
                    Dim Value As String = ""
                    If Not (table.Item(keysArray(count)) Is Nothing) Then
                        Value = table.Item(keysArray(count)).ToString()
                    End If
                    System.Console.WriteLine("Property " + keysArray(count).ToString() + " = " + Value)
                Next count
            End If
        End Sub

        ' Print the properties of a TestObject as an array.
        Public Overridable Sub PrintPropertiesAsArray(ByVal tobj As TestObject)
            Dim table As System.Collections.Hashtable = tobj.GetProperties()
            If Not (table.Keys Is Nothing) Then
                Dim keysArray(table.Keys.Count) As System.Object
                table.Keys.CopyTo(keysArray, 0)
                Dim count As Integer
                For count = 0 To table.Keys.Count - 1
                    System.Console.WriteLine(Microsoft.VisualBasic.ControlChars.Quote + keysArray(count).ToString() + Microsoft.VisualBasic.ControlChars.Quote + ",")
                Next count
            End If
        End Sub

        Protected Class ScriptPrerequisiteNotMetException
            Inherits Exception
            Private Extra As Object
            Sub New(ByVal Message As String, ByVal Extra As Object)
                MyBase.New(Message)
                Me.Extra = Extra
            End Sub

            Function GetExtraObject() As Object
                Return Me.Extra
            End Function
        End Class

        Public Overrides Function OnUnhandledException(ByVal e As Exception) As Boolean
            If TypeOf e Is ScriptPrerequisiteNotMetException Then
                Dim spe As ScriptPrerequisiteNotMetException = e
                OnScriptPrerequisiteNotMet(spe.Message, spe.GetExtraObject())
                Return True
            End If
            Return False
        End Function

        Public Function OnScriptPrerequisiteNotMet(ByVal Message As String, ByVal Extra As Object) As Boolean
            LogWarning("Script prerequisite not met: " + Message)
        End Function

        Protected Sub ScriptRequiresApp(ByVal AppName As String)
            If Not IsApplicationDefined(AppName) Then
                Throw New ScriptPrerequisiteNotMetException("Application not defined: " + AppName, Nothing)
            End If
        End Sub

        Protected Sub ScriptRequiresLanguage(ByVal LangCode As String)
            Dim CurrentLanguage As String = System.Globalization.CultureInfo.CurrentCulture.TwoLetterISOLanguageName
            If Not LangCode.Equals(CurrentLanguage) Then
                Throw New ScriptPrerequisiteNotMetException("Current Locale is: " + CurrentLanguage + ", it should be: " + LangCode, Nothing)
            End If
        End Sub

        Public Sub CleanupAndLogException(ByVal pto As ProcessTestObject, ByVal e As System.Exception)

            Try

                If pto.IsAlive() Then

                    pto.Kill()
                End If

            Catch

            End Try
            LogException(e)
        End Sub
    End Class 'ExtensionScript
End Namespace 'SuperScript
